"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.convertTypeAnnotationToFlowType = convertTypeAnnotationToFlowType;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function flowTypeAnnotationToString(type, nullable, fieldName) {
  switch (type.type) {
    case "BooleanTypeAnnotation":
      return { type: "boolean", nullable: nullable, fieldName: fieldName };
    case "NumberTypeAnnotation":
      return { type: "number", nullable: nullable, fieldName: fieldName };
    case "StringTypeAnnotation":
      return { type: "string", nullable: nullable, fieldName: fieldName };
    default:
      return { type: "any", nullable: nullable, fieldName: fieldName };
  }
}

// convert an ObjectTypeAnnotation to a js object

function convertFlowObjectTypeAnnotation(objectType, flowTypes, fieldName) {
  return objectType.properties.reduce(function (obj, property) {
    var key = property.key.name;
    // property.optional indicates the presence of a question mark at the end of a field
    // eg. field?: blah
    // given a GraphQL schema cannot represent an optional object key we don't need to care about it
    var nullable = false;

    return _extends({}, obj, _defineProperty({}, key, convertTypeAnnotationToFlowType(property.value, nullable, flowTypes, fieldName)));
  }, {});
}

function convertTypeAnnotationToFlowType(value, nullable, flowTypes, fieldName) {
  if (value.type === "NullableTypeAnnotation") {
    return convertTypeAnnotationToFlowType(value.typeAnnotation, true, flowTypes, fieldName);
  }

  if (value.type === "GenericTypeAnnotation") {
    if (flowTypes[value.id.name]) {
      return convertTypeAnnotationToFlowType(flowTypes[value.id.name], nullable, flowTypes, fieldName);
    }

    if (value.typeParameters) {
      if (value.id.name === "AliasFor") {
        var _value$typeParameters = _slicedToArray(value.typeParameters.params, 2),
            fieldNameType = _value$typeParameters[0],
            type = _value$typeParameters[1];

        return convertTypeAnnotationToFlowType(type, nullable, flowTypes, fieldNameType.value);
      }
    }
  }

  if (value.type === "ObjectTypeAnnotation") {
    return {
      fieldName: fieldName,
      type: "object",
      nullable: nullable,
      properties: convertFlowObjectTypeAnnotation(value, flowTypes)
    };
  }

  if (value.type === "ArrayTypeAnnotation") {
    return {
      type: "array",
      child: convertTypeAnnotationToFlowType(value.elementType, nullable, flowTypes, fieldName)
    };
  }

  return flowTypeAnnotationToString(value, nullable, fieldName);
}